//
//  AnalogClockView.swift
//  Analog Clock
//
//  Created by Holger Hinzberg on 18.01.16.
//  Copyright © 2016 Holger Hinzberg. All rights reserved.
//

import UIKit

class AnalogClockView: UIView
{
    override init(frame: CGRect)
    {
        super.init(frame: frame)
        self.backgroundColor = UIColor.whiteColor()
    }
    
    required init?(coder aDecoder: NSCoder)
    {
        super.init(coder: aDecoder)
    }
    
    // Only override drawRect: if you perform custom drawing.
    // An empty implementation adversely affects performance during animation.
    override func drawRect(rect: CGRect)
    {
        // Den grafischen Context holen
        let context = UIGraphicsGetCurrentContext()
        
        // Den Hintergrund zeichnen
        self.drawClockBackgroundWithContext(context!, size: self.bounds.size)
        // Das Zifferblatt zeichnen
        self.drawClockFaceWithContext(context!, size: self.bounds.size)
        // Die drei Zeiger der Uhr zeichnen
        self.drawClockHandsWithContext(context!, size: self.bounds.size)
    }
    
    func drawClockBackgroundWithContext(
        context:CGContextRef, size:CGSize)
    {
        let height = size.height
        let width = size.width
        let minLength = fmin(height, width)
        
        // Die Linien- und die Füllfarbe setzen
        let fillColor:[CGFloat] = [0.9, 0.9, 0.9, 1.0]
        let strokeColor:[CGFloat] = [0.5, 0.5, 0.5, 1.0]
        CGContextSetStrokeColor(context, strokeColor)
        CGContextSetFillColor(context, fillColor)
        
        // Die Linienstärke setzen
        CGContextSetLineWidth(context, 2)
        
        // Den Kreis im View zentrieren
        let clockDiameter = minLength - 5
        let distanceLeft = (width - clockDiameter) / 2
        let distanceTop = (height - clockDiameter) / 2
        
        let borderRect = CGRectMake(distanceLeft, distanceTop, clockDiameter, clockDiameter)
        
        // Den Kreis zeichnen
        CGContextFillEllipseInRect(context, borderRect)
        CGContextStrokeEllipseInRect(context, borderRect)
    }
    
    func drawClockFaceWithContext(
        context:CGContextRef, size:CGSize)
    {
        let height = size.height
        let width = size.width
        let minLength = fmin(height, width)
        
        let radiusOuter:CGFloat = (minLength / 2) - 5
        let radiusInner:CGFloat = radiusOuter - 15
        
        let centerPoint
          = CGPointMake(size.width / 2, size.height / 2)
        
        // Die Linienstärke und -farbe konfigurieren
        CGContextSetLineWidth(context, 5)
        let blackColor:[CGFloat] = [0.0, 0.0, 0.0, 1.0]
        CGContextSetStrokeColor(context, blackColor)
        
        // Das Zifferblatt zeichnen
        for var i:Double = 0; i < 360; i = i + 30
         {
            let angle = i / 180.0 * M_PI
            
            let innerPoint
            = CGPointMake(radiusInner * CGFloat(cos(angle))
            + centerPoint.x, radiusInner * CGFloat(sin(angle))
                + centerPoint.y)
            
            let outerPoint
            = CGPointMake(radiusOuter * CGFloat(cos(angle))
                + centerPoint.x, radiusOuter * CGFloat(sin(angle))
                    + centerPoint.y)
            
            CGContextMoveToPoint(context, innerPoint.x, innerPoint.y)
            CGContextAddLineToPoint(context, outerPoint.x, outerPoint.y)
            
            CGContextStrokePath(context)
        }
        
        // Einen kleinen Kreis in der Mitte des View zeichnen
        let clockDiameter:CGFloat = 18
        let distanceLeft = (width - clockDiameter) / 2
        let distanceTop = (height - clockDiameter) / 2
        
        let borderRect
        = CGRectMake(distanceLeft, distanceTop,
            clockDiameter, clockDiameter)
        
        CGContextSetFillColor(context, blackColor)
        CGContextFillEllipseInRect(context, borderRect)
    }
    
    func drawClockHandsWithContext(
        context:CGContextRef, size:CGSize)
    {
        // Die einzelnen Teile der aktuelle Uhrzeit ermitteln
        let cal = NSCalendar(calendarIdentifier: "gregorian")
        let now = NSDate()
        let hour = cal!.component(NSCalendarUnit.Hour, fromDate: now)
        let minute = cal!.component(NSCalendarUnit.Minute, fromDate: now)
        let second = cal!.component(NSCalendarUnit.Second, fromDate: now)
        
        // Die Stunden, Minuten und Sekunden in Winkel umrechnen
        var angleHours = Double(hour * 30 + 30 * minute / 60 - 90)
        angleHours = angleHours / 180.0 * M_PI
        
        var angleMinutes = Double(minute * 6 - 90)
        angleMinutes = angleMinutes / 180.0 * M_PI
        
        var angleSeconds = Double(second * 6 - 90)
        angleSeconds = angleSeconds / 180.0 * M_PI
        
        let height = size.height
        let width = size.width
        let minLength = fmin(height, width)
        
        // Die Zeiger in verschiedenen Linienstärken zeichnen
        let blackColor:[CGFloat] = [0.0, 0.0, 0.0, 1.0]
        CGContextSetStrokeColor(context, blackColor)
        var handLength = CGFloat(minLength) / 2.0 - 30
        self.drawLineWithContext(context, size: size,
            thickness: 8.0, angle: angleHours, length: handLength)
        handLength = CGFloat(minLength) / 2.0 - 15
        self.drawLineWithContext(context, size: size,
            thickness: 5.0, angle: angleMinutes, length: handLength)
        handLength = CGFloat(minLength) / 2.0 - 12
        self.drawLineWithContext(context, size: size,
            thickness: 5.0, angle: angleSeconds, length: handLength)
    }
    
    func drawLineWithContext(
        context:CGContextRef, size:CGSize, thickness:CGFloat, angle:Double, length:CGFloat)
    {
        CGContextSetLineWidth(context, thickness)
        
        let centerPoint
        = CGPointMake(size.width / 2.0, size.height / 2.0)
        
        let endPoint
        = CGPointMake(length * CGFloat(cos(angle))
        + centerPoint.x, length * CGFloat(sin(angle)) + centerPoint.y)
        
        CGContextMoveToPoint(context, centerPoint.x, centerPoint.y)
        CGContextAddLineToPoint(context, endPoint.x, endPoint.y)
        CGContextStrokePath(context)
    }
    
    func update()
    {
        self.setNeedsDisplay()
    }
    
    
}
